/*
 * This file is part of ReporteRs
 * Copyright (c) 2014, David Gohel All rights reserved.
 * This program is licensed under the GNU GENERAL PUBLIC LICENSE V3.
 * You may obtain a copy of the License at :
 * http://www.gnu.org/licenses/gpl.html
 */

package org.lysis.reporters.formats;

import java.util.LinkedHashMap;

import org.docx4j.dml.CTTableCellProperties;
import org.docx4j.dml.STTextAnchoringType;
import org.docx4j.wml.CTShd;
import org.docx4j.wml.CTVerticalJc;
import org.docx4j.wml.STVerticalJc;
import org.docx4j.wml.TcPr;
import org.docx4j.wml.TcPrInner.TcBorders;


public class CellProperties {
	private BorderProperties borderBottom;
	private BorderProperties borderLeft;
	private BorderProperties borderTop;
	private BorderProperties borderRight;
	private String verticalAlign;
	private int paddingBottom;
	private int paddingTop;
	private int paddingLeft;
	private int paddingRight;
	private String backgroundColor;
	
	public CellProperties(BorderProperties borderBottom,
			BorderProperties borderLeft, BorderProperties borderTop,
			BorderProperties borderRight, String verticalAlign,
			int paddingBottom, int paddingTop, int paddingLeft,
			int paddingRight, String backgroundColor) {
		super();
		this.borderBottom = borderBottom;
		this.borderLeft = borderLeft;
		this.borderTop = borderTop;
		this.borderRight = borderRight;
		this.verticalAlign = verticalAlign;
		this.paddingBottom = paddingBottom;
		this.paddingTop = paddingTop;
		this.paddingLeft = paddingLeft;
		this.paddingRight = paddingRight;
		this.backgroundColor = backgroundColor;
	}


	
	public String getVerticalAlign() {
		return verticalAlign;
	}
	public int getPaddingBottom() {
		return paddingBottom;
	}
	public int getPaddingTop() {
		return paddingTop;
	}
	public int getPaddingLeft() {
		return paddingLeft;
	}
	public int getPaddingRight() {
		return paddingRight;
	}
	public String getBackgroundColor() {
		return backgroundColor;
	}
	public void setBackgroundColor(String color) {
		backgroundColor = color;
	}


	public BorderProperties getBorderBottom() {
		return borderBottom;
	}



	public void setBorderBottom(BorderProperties borderBottom) {
//		if( Debug.debug ) System.err.println("{cellProperties}\tsetBorderBottom" );
		this.borderBottom = borderBottom;
	}



	public BorderProperties getBorderLeft() {
		return borderLeft;
	}



	public void setBorderLeft(BorderProperties borderLeft) {
//		if( Debug.debug ) System.err.println("{cellProperties}\tsetBorderLeft" );
		this.borderLeft = borderLeft;
	}



	public BorderProperties getBorderTop() {
		return borderTop;
	}



	public void setBorderTop(BorderProperties borderTop) {
//		if( Debug.debug ) System.err.println("{cellProperties}\tsetBorderTop" );
		this.borderTop = borderTop;
	}



	public BorderProperties getBorderRight() {
		return borderRight;
	}



	public void setBorderRight(BorderProperties borderRight) {
//		if( Debug.debug ) System.err.println("{cellProperties}\tsetBorderRight" );
		this.borderRight = borderRight;
	}

	public CellProperties getClone(){
		return( new CellProperties( borderBottom,
				 borderLeft,  borderTop,
				 borderRight,  verticalAlign,
				 paddingBottom,  paddingTop,  paddingLeft,
				 paddingRight,  backgroundColor) );
	}
	
	public TcPr get_docx_cp() {
		TcPr tcPr = new TcPr();
		
		CTShd shd = new CTShd();
		shd.setFill(backgroundColor);
		tcPr.setShd(shd);
		TcBorders tcb = new TcBorders();
	    tcb.setBottom(borderBottom.get_docx_bd());
	    
	    tcb.setLeft(borderLeft.get_docx_bd());
	    tcb.setRight(borderRight.get_docx_bd());
	    tcb.setTop(borderTop.get_docx_bd());
	    tcPr.setTcBorders(tcb);
		
	    CTVerticalJc valign = new CTVerticalJc();
	    if( verticalAlign.equals("center") )
	    	valign.setVal(STVerticalJc.CENTER);
	    else if( verticalAlign.equals("middle") )
	    	valign.setVal(STVerticalJc.CENTER);
	    else if( verticalAlign.equals("top") )
	    	valign.setVal(STVerticalJc.TOP);
	    else if( verticalAlign.equals("bottom") )
	    	valign.setVal(STVerticalJc.BOTTOM);
	    else valign.setVal(STVerticalJc.CENTER);
	    tcPr.setVAlign(valign);
		
	    tcPr.setTcMar(org.lysis.reporters.docx4r.tools.Format.getMargins(paddingBottom, paddingTop, paddingLeft, paddingRight) );

	    return tcPr;
	}
	public CTTableCellProperties get_pptx_cp() throws Exception {
		CTTableCellProperties tcPr = new CTTableCellProperties();
		
		tcPr.setSolidFill(org.lysis.reporters.pptx4r.tools.Format.getCol(backgroundColor));
		tcPr.setLnB(borderBottom.get_pptx_bd("B"));
		tcPr.setLnT(borderTop.get_pptx_bd("T"));
		tcPr.setLnR(borderRight.get_pptx_bd("R"));
		tcPr.setLnL(borderLeft.get_pptx_bd("L"));
				
	    if( verticalAlign.equals("center") )
	    	tcPr.setAnchor(STTextAnchoringType.CTR);
	    else if( verticalAlign.equals("middle") )
	    	tcPr.setAnchor(STTextAnchoringType.CTR);
	    else if( verticalAlign.equals("top") )
	    	tcPr.setAnchor(STTextAnchoringType.T);
	    else if( verticalAlign.equals("bottom") )
	    	tcPr.setAnchor(STTextAnchoringType.B);
	    else tcPr.setAnchor(STTextAnchoringType.CTR);
	    tcPr.setMarB(new Integer(paddingBottom*12700));
	    tcPr.setMarT(new Integer(paddingTop*12700));
	    tcPr.setMarR(new Integer(paddingRight*12700));
	    tcPr.setMarL(new Integer(paddingLeft*12700));

	    return tcPr;
	}
	
	public LinkedHashMap<String, String> get_html_cp( )  {
		LinkedHashMap<String, String> tcPr = new LinkedHashMap<String, String>();

		tcPr.put("background-color", backgroundColor);	

		tcPr.put("border-bottom-color", borderBottom.getBorderColor() );	
		tcPr.put("border-bottom-style", borderBottom.getBorderStyle() );	
		tcPr.put("border-bottom-width", borderBottom.getBorderWidth() + "px" );	
		tcPr.put("border-top-color", borderTop.getBorderColor() );	
		tcPr.put("border-top-style", borderTop.getBorderStyle() );	
		tcPr.put("border-top-width", borderTop.getBorderWidth() + "px" );	
		tcPr.put("border-right-color", borderRight.getBorderColor() );	
		tcPr.put("border-right-style", borderRight.getBorderStyle() );	
		tcPr.put("border-right-width", borderRight.getBorderWidth() + "px" );	
		tcPr.put("border-left-color", borderLeft.getBorderColor() );	
		tcPr.put("border-left-style", borderLeft.getBorderStyle() );	
		tcPr.put("border-left-width", borderLeft.getBorderWidth() + "px" );	

		if( verticalAlign.equals("center") )
			tcPr.put("vertical-align", "middle" );	
		else tcPr.put("vertical-align", verticalAlign );	
		
	
		tcPr.put("padding-left", paddingLeft + "px" );	
		tcPr.put("padding-right", paddingRight + "px" );	
		tcPr.put("padding-top", paddingTop + "px" );	
		tcPr.put("padding-bottom", paddingBottom + "px" );	

	    return tcPr;
	}
	
}
